@@ -188,37 +188,7 @@ class SETSignalCreator:
 
         if timeframe == fld.TIMEFRAME_DAILY:
             if value_by == fld.VALUE_BY_STOCK:
-                df = self._sdr.get_data_symbol_daily(
-                    field=field,
-                    symbol_list=symbol_list,
-                    start_date=self._start_date,
-                    end_date=self._end_date,
-                )
-
-                # Forward fill 0 and NaN with prior
-                if field in {
-                    fld.D_OPEN,
-                    fld.D_HIGH,
-                    fld.D_LOW,
-                    fld.D_CLOSE,
-                    fld.D_AVERAGE,
-                    fld.D_LAST_BID,
-                    fld.D_LAST_OFFER,
-                }:
-                    prior_df = self._sdr.get_data_symbol_daily(
-                        field=fld.D_PRIOR,
-                        symbol_list=symbol_list,
-                        start_date=self._start_date,
-                        end_date=self._end_date,
-                    )
-                    prior_df = self._reindex_trade_date(prior_df)
-                    prior_df = prior_df.replace(0, np.nan)
-                    prior_df = prior_df.fillna(method="ffill")
-
-                    df = self._reindex_trade_date(df)
-                    df = df.replace(0, np.nan)
-                    df = df.fillna(prior_df)
-
+                df = self._get_data_symbol_daily(symbol_list=symbol_list, field=field)
             elif value_by == fld.VALUE_BY_SECTOR:
                 df = self._sdr._get_daily_sector_info_by_security(
                     field=field,
@@ -260,34 +230,20 @@ class SETSignalCreator:
             fld.TIMEFRAME_YTD,
         ):
             if value_by == fld.VALUE_BY_STOCK:
-                df = self._sdr._get_fundamental_data(
-                    field=field,
+                df = self._get_data_fundamental_symbol(
                     symbol_list=symbol_list,
-                    start_date=self._start_date,
-                    end_date=self._end_date,
+                    field=field,
                     timeframe=timeframe,
-                    fillna_value=np.inf,
+                    method=method,
+                    method_args=method_args,
+                    method_kwargs=method_kwargs,
+                    period=period,
+                    shift=shift,
                 )
-
-                df = df.apply(
-                    lambda x: self._rolling_skip_na_keep_inf(
-                        x,
-                        method=method,
-                        period=period,
-                        shift=shift,
-                        args=method_args,
-                        kwargs=method_kwargs,
-                    ),
-                    axis=0,
-                )
-                df = self._reindex_trade_date(df)
-                df = df.fillna(method="ffill")
-                df = df.replace(np.inf, np.nan)
             else:
                 raise InputError(
                     f"{value_by} is invalid value_by. Please read document to check valid value_by."
                 )
-
         else:
             raise InputError(
                 f"{timeframe} is invalid timeframe. Please read document to check valid timeframe."
@@ -339,6 +295,8 @@ class SETSignalCreator:
         except InputError:
             return self._is_universe_dynamic(universe)
 
+    # TODO: remove cache after improve SETSignalCreator._is_banned_sp
+    @lru_cache(maxsize=1)
     def is_banned(self) -> pd.DataFrame:
         """Return True when stock was Delisted or Suspension (SP).
 
@@ -539,6 +497,76 @@ class SETSignalCreator:
             end_date=self._end_date,
         )
 
+    def _get_data_symbol_daily(self, symbol_list: List[str], field: str):
+        df = self._sdr.get_data_symbol_daily(
+            field=field,
+            symbol_list=symbol_list,
+            start_date=self._start_date,
+            end_date=self._end_date,
+        )
+
+        # Forward fill 0 and NaN with prior
+        if field in {
+            fld.D_OPEN,
+            fld.D_HIGH,
+            fld.D_LOW,
+            fld.D_CLOSE,
+            fld.D_AVERAGE,
+            fld.D_LAST_BID,
+            fld.D_LAST_OFFER,
+        }:
+            prior_df = self._sdr.get_data_symbol_daily(
+                field=fld.D_PRIOR,
+                symbol_list=symbol_list,
+                start_date=self._start_date,
+                end_date=self._end_date,
+            )
+            prior_df = self._reindex_trade_date(prior_df)
+            prior_df = prior_df.replace(0, np.nan)
+            prior_df = prior_df.fillna(method="ffill")
+
+            df = self._reindex_trade_date(df)
+            df = df.replace(0, np.nan)
+            df = df.fillna(prior_df)
+        return df
+
+    def _get_data_fundamental_symbol(
+        self,
+        symbol_list: List[str],
+        field: str,
+        timeframe: str,
+        method: str,
+        method_args: Optional[tuple],
+        method_kwargs: Optional[dict],
+        period: int,
+        shift: int,
+    ):
+        df = self._sdr._get_fundamental_data(
+            field=field,
+            symbol_list=symbol_list,
+            start_date=self._start_date,
+            end_date=self._end_date,
+            timeframe=timeframe,
+            fillna_value=np.inf,
+        )
+
+        df = df.apply(
+            lambda x: self._rolling_skip_na_keep_inf(
+                x,
+                method=method,
+                period=period,
+                shift=shift,
+                args=method_args,
+                kwargs=method_kwargs,
+            ),
+            axis=0,
+        )
+        df = self._reindex_trade_date(df)
+        df = df.fillna(method="ffill")
+        df = df.replace(np.inf, np.nan)
+
+        return df
+
     def _is_universe_static(self, universe: str) -> pd.DataFrame:
         if universe in fld.MARKET_MAP_UPPER:
             index_type = "market"
